/*
 *      navqueue.h
 *      
 *      Copyright 2007 dave <dave@poonix>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "geany.h"

#include "navqueue.h"
#include "sciwrappers.h"
#include "document.h"
#include "utils.h"


void navqueue_init(void)
{
	navigation_queue = g_queue_new();
	nav_queue_pos = 0;
}

void navqueue_free(void)
{
	while (! g_queue_is_empty(navigation_queue))
	{
		g_free(g_queue_pop_tail(navigation_queue));
	}
	g_queue_free(navigation_queue);
}

void adjust_buttons(void)
{
	if (g_queue_get_length(navigation_queue) < 2)
	{
		gtk_widget_set_sensitive(app->navigation_buttons[0], FALSE);
		gtk_widget_set_sensitive(app->navigation_buttons[1], FALSE);
		return;
	}
	if (nav_queue_pos == 0)
	{
		gtk_widget_set_sensitive(app->navigation_buttons[0], TRUE);
		gtk_widget_set_sensitive(app->navigation_buttons[1], FALSE);
		return;
	}
	// forward should be sensitive since where not at the start
	gtk_widget_set_sensitive(app->navigation_buttons[1], TRUE);
	
	// back should be sensitive if theres a place to go back to
	(nav_queue_pos < g_queue_get_length(navigation_queue)-1) ?
		gtk_widget_set_sensitive(app->navigation_buttons[0], TRUE) :
			gtk_widget_set_sensitive(app->navigation_buttons[0], FALSE);
}

void navqueue_new_position(gchar *file, gint line)
{
	filepos *npos;
	guint i;
	
	npos = g_new0(filepos, 1);
	npos->file = file;
	npos->line = line;
	
	// if weve jumped to a new position from 
	// inside the the queue rather than going forward
	if (nav_queue_pos > 0) 
	{
		for (i = 0; i < nav_queue_pos; i++)
		{
			g_free(g_queue_pop_head(navigation_queue));
		}
		nav_queue_pos = 0;
	}
	g_queue_push_head(navigation_queue, npos);
	adjust_buttons();
}

void navqueue_go_back(void)
{
	filepos *fprev;
	
	// return if theres no place to go back to
	if (g_queue_is_empty(navigation_queue) || 
		nav_queue_pos >= g_queue_get_length(navigation_queue)-1)
		return;
		
	// jump back
	fprev = g_queue_peek_nth(navigation_queue, nav_queue_pos+1);
	if (utils_goto_file_line(fprev->file, TRUE, fprev->line))
	{
		nav_queue_pos++;
	}
	else
	{
		//TODO: add option to re open the file
		g_queue_pop_nth(navigation_queue, nav_queue_pos+1);
	}
	adjust_buttons();
}

void navqueue_go_forward(void)
{
	filepos *fnext;

	if (nav_queue_pos < 1 || g_queue_get_length(navigation_queue) < 2)
		return;

	// jump forward
	fnext = g_queue_peek_nth(navigation_queue, nav_queue_pos-1);
	if (utils_goto_file_line(fnext->file, TRUE, fnext->line))
	{
		nav_queue_pos--;
	}
	else
	{
		//TODO: add option to re open the file
		g_queue_pop_nth(navigation_queue, nav_queue_pos-1);
	}
	
	adjust_buttons();
}

